use dotenv::dotenv;
use reqwest::{self, multipart};
use serde_json::{json, Value};
use std::env;
use std::error::Error;
use std::path::{Path, PathBuf};
use tokio;
use base64::{Engine as _, engine::general_purpose::STANDARD};

#[derive(Debug)]
struct Config {
    wp_url: String,
    wp_username: String,
    wp_app_password: String,
    openai_api_key: String,
    openai_api_url: String,
    openai_model: String,
}

impl Config {
    fn from_env() -> Result<Self, Box<dyn Error>> {
        dotenv()?;
        
        Ok(Config {
            wp_url: env::var("WP_URL")?,
            wp_username: env::var("WP_USERNAME")?,
            wp_app_password: env::var("WP_APP_PASSWORD")?,
            openai_api_key: env::var("OPENAI_API_KEY")?,
            openai_api_url: env::var("OPENAI_API_URL")?,
            openai_model: env::var("OPENAI_MODEL")?,
        })
    }
}

fn validate_image_path(path_str: &str) -> Result<PathBuf, Box<dyn Error>> {
    let path = PathBuf::from(path_str);
    
    // Check if path exists
    if !path.exists() {
        return Err(format!("Image path does not exist: {}", path_str).into());
    }

    // Check if it's a file
    if !path.is_file() {
        return Err(format!("Path is not a file: {}", path_str).into());
    }

    // Check file extension
    match path.extension().and_then(|ext| ext.to_str()) {
        Some(ext) => {
            let lower_ext = ext.to_lowercase();
            if !["jpg", "jpeg", "png", "gif"].contains(&lower_ext.as_str()) {
                return Err(format!("Unsupported file type: {}", ext).into());
            }
        }
        None => return Err("File has no extension".into()),
    }

    // Try to canonicalize the path (resolve symlinks and normalize)
    let canonical_path = path.canonicalize()
        .map_err(|e| format!("Failed to resolve path: {}", e))?;

    println!("Using image file: {}", canonical_path.display());
    Ok(canonical_path)
}

#[tokio::main]
async fn main() -> Result<(), Box<dyn Error>> {
    // Get image path from command line arguments
    let args: Vec<String> = std::env::args().collect();
    if args.len() != 2 {
        eprintln!("Usage: {} <image_path>", args[0]);
        std::process::exit(1);
    }
    
    // Validate and normalize the image path
    let image_path = validate_image_path(&args[1])?;

    // Load configuration
    let config = Config::from_env()?;

    // Get image description from ChatGPT
    let description = get_image_description(&config, &image_path).await?;
    println!("Generated description: {}", description);

    // Upload image to WordPress and set ALT text
    upload_to_wordpress(&config, &image_path, &description).await?;

    Ok(())
}

async fn get_image_description(config: &Config, image_path: &Path) -> Result<String, Box<dyn Error>> {
    // Read and encode image
    let image_data = tokio::fs::read(image_path)
        .await
        .map_err(|e| format!("Failed to read image file: {}", e))?;
    
    let base64_image = STANDARD.encode(&image_data);

    // Create ChatGPT API request
    let client = reqwest::Client::new();
    let response = client
        .post(&config.openai_api_url)
        .header("Authorization", format!("Bearer {}", config.openai_api_key))
        .header("Content-Type", "application/json")
        .json(&json!({
            "model": config.openai_model,
            "max_tokens": 300,
            "messages": [
                {
                    "role": "user",
                    "content": [
                        {
                            "type": "text",
                            "text": "Please describe this image concisely for use as an alt text description. Focus on key visual elements and context."
                        },
                        {
                            "type": "image_url",
                            "image_url": {
                                "url": format!("data:image/jpeg;base64,{}", base64_image)
                            }
                        }
                    ]
                }
            ]
        }))
        .send()
        .await?;

    // Improved error handling for API response
    if !response.status().is_success() {
        let error_text = response.text().await?;
        return Err(format!("OpenAI API error: {}", error_text).into());
    }

    let result: Value = response.json().await?;
    
    // More detailed error handling for JSON parsing
    let description = result["choices"]
        .get(0)
        .ok_or("No choices in response")?
        ["message"]["content"]
        .as_str()
        .ok_or("Invalid content format in response")?
        .to_string();

    Ok(description)
}

async fn upload_to_wordpress(
    config: &Config,
    image_path: &Path,
    description: &str,
) -> Result<(), Box<dyn Error>> {
    let client = reqwest::Client::new();
    let file_data = tokio::fs::read(image_path)
        .await
        .map_err(|e| format!("Failed to read image for upload: {}", e))?;
        
    let filename = image_path
        .file_name()
        .ok_or("Invalid filename")?
        .to_str()
        .ok_or("Invalid UTF-8 in filename")?;

    // Create form with image file
    let part = multipart::Part::bytes(file_data)
        .file_name(filename.to_string())
        .mime_str("image/jpeg")?;

    let form = multipart::Form::new()
        .part("file", part)
        .text("alt_text", description.to_string())
        .text("description", description.to_string());

    // Upload to WordPress
    let response = client
        .post(format!("{}/wp-json/wp/v2/media", config.wp_url))
        .basic_auth(&config.wp_username, Some(&config.wp_app_password))
        .multipart(form)
        .send()
        .await?;

    let status = response.status();
    if status.is_success() {
        let response_json: Value = response.json().await?;
        println!("Successfully uploaded image to WordPress with description");
        if let Some(url) = response_json["source_url"].as_str() {
            println!("Image URL: {}", url);
        }
    } else {
        let error_text = response.text().await?;
        println!("Failed to upload image: {}", status);
        println!("Response: {}", error_text);
    }

    Ok(())
}
